#include "RelocHack.h"
#include "CommToolImpl.h"
#include "RFCOMMTool.h"
#include "BluntServer.h"
#include "HCI.h"
#include "L2CAP.h"
#include <stdarg.h>
#include <stdio.h>

extern "C" _sys_write (int fd, char* data, int len);

TRFCOMMTool::TRFCOMMTool (unsigned long serviceId):
	TCommTool (serviceId)
{
	fLogLevel = 0;
	fSavedData = new UByte[MAX_SAVE];
	
	HLOG (0, "-------------------------\nTRFCOMMTool\n");
	
	RelocVTable (__VTABLE__11TRFCOMMTool);
}

TRFCOMMTool::~TRFCOMMTool (void)
{
	HLOG (0, "~TRFCOMMTool\n");
	delete fSavedData;
}

#pragma mark -

// ================================================================================
//  Comm Tool Functions
// ================================================================================

NewtonErr TRFCOMMTool::HandleRequest (TUMsgToken& msgToken, ULong msgType)
{
	NewtonErr r;
	BluntEvent* event;
	
	HLOG (1, "TRFCOMMTool::HandleRequest %08x %08x\n", &msgToken, msgType);
	if (msgType == BLUNT_MSG_TYPE) {
		event = (BluntEvent *) fMsgData;
		HLOG (1, "  Blunt event: %08x %08x %d\n", event, event->fOriginalEvent, event->fType);
		switch (event->fType) {
			case E_CONNECTION_COMPLETE: {
				BluntConnectionCompleteEvent *e = (BluntConnectionCompleteEvent *) event;
				ConnectComplete (e->fResult);
				if (e->fResult == noErr) {
					fHCIHandle = e->fHCIHandle;
					fPeerRFCOMMPort = e->fRFCOMMPort;
					fDataHandler = e->fHandler;
					LOG	("  Connection complete: %04x\n", fHCIHandle);
				} else {
					LOG	("  Connection error: %d\n", e->fResult);
				}
				} break;
			case E_DATA: {
				BluntDataEvent *e = (BluntDataEvent *) event;
				Size n;
				if (e->fResult == noErr) {
					if (fGetBuffer != NULL) {
						n = fGetBuffer->Putn (e->fData, e->fLength);
						if (fGetBuffer->AtEOF ()) GetComplete (noErr);
						HLOG (1, "  Data: %d %d\n", n, e->fLength);
						if (e->fLength > n) {
							memcpy (fSavedData + fSavedDataAmount, e->fData + n, e->fLength - n);
							fSavedDataAmount += (e->fLength - n);
						}
						if (e->fDelete) delete e->fData;
					}
				} else {
					GetComplete (e->fResult);
				}
				} break;
			case E_DATA_SENT: {
				BluntDataSentEvent *e = (BluntDataSentEvent *) event;
				fDataSent += e->fAmount;
				if (!fPutBuffer->AtEOF () > 0 && e->fResult == noErr && e->fAmount > 0) {
					SendPendingData ();
				} else {
					PutComplete (e->fResult, fDataSent);
				}
				} break;
			case E_DISCONNECT_COMPLETE:
				TerminateComplete ();
				break;
		}
//		delete event->fOriginalEvent;
		r = noErr;
	} else {
		r = TCommTool::HandleRequest (msgToken, msgType);
	}
	return r;
}

NewtonErr TRFCOMMTool::ProcessOptionStart (TOption* theOption, ULong label, ULong opcode)
{
	Char s[5];
	
	memset (s, 0, 5);
	memcpy (s, &label, 4);
	HLOG (1, "TRFCOMMTool::ProcessOptionStart: %s\n", s);
	
	if (label == 'addr') {
		TRFCOMMAddressOption *o;
		o = (TRFCOMMAddressOption *) theOption;
		fPeerBdAddr[0] = o->fBdAddr[5]; fPeerBdAddr[1] = o->fBdAddr[4]; 
		fPeerBdAddr[2] = o->fBdAddr[3]; fPeerBdAddr[3] = o->fBdAddr[2]; 
		fPeerBdAddr[4] = o->fBdAddr[1]; fPeerBdAddr[5] = o->fBdAddr[0]; 
		fPeerRFCOMMPort = o->fPort;
		HLOG (1, "  Peer Address: %02x:%02x:%02x:", o->fBdAddr[0], o->fBdAddr[1], o->fBdAddr[2]);
		HLOG (1, "%02x:%02x:%02x ", o->fBdAddr[3], o->fBdAddr[4], o->fBdAddr[5]);
		HLOG (1, "%d\n", o->fPort);
	} else if (label == 'pinc') {
		TRFCOMMPINCodeOption *o;
		o = (TRFCOMMPINCodeOption *) theOption;
		memcpy (fPINCode, o->fPINCode, sizeof (fPINCode));
		fPINCode[o->fPINCodeLength] = 0;
		HLOG (1, "  PIN Code: %s\n", fPINCode);
	}
	return TCommTool::ProcessOptionStart (theOption, label, opcode);
}

void TRFCOMMTool::BindStart ()
{
	TUNameServer nameServer;
	ULong id;
	ULong spec;
	HLOG (1, "TRFCOMMTool::BindStart\n");
	TObjectId myPort;
	
	nameServer.Lookup ("BluntServer", "TUPort", &id, &spec);
	fServerPort = id;
	fGetBuffer = NULL;
	myPort = * (TObjectId*) ( (Byte*) this + 0x8c);
	HLOG (1, "  Port: %04x, Server port: %04x\n", myPort, id);
	BindComplete (noErr);
}

void TRFCOMMTool::ConnectStart ()
{
	BluntConnectionCommand* command = new BluntConnectionCommand ();
	
	HLOG (1, "TRFCOMMTool::ConnectStart\n");
	fSavedDataAmount = 0;
	command->fToolPort = * (TObjectId*) ( (Byte*) this + 0x8c);
	memcpy (command->fBdAddr, fPeerBdAddr, 6);
	command->fTargetLayer = L_RFCOMM;
	command->fL2CAPProtocol = PSM_RFCOMM;
	command->fRFCOMMPort = fPeerRFCOMMPort;
	fServerPort.Send (command, sizeof (*command), kNoTimeout, M_COMMAND);
}

void TRFCOMMTool::ListenStart (void)
{
	HLOG (1, "TRFCOMMTool::ListenStart\n");
	ListenComplete (noErr);
}

void TRFCOMMTool::AcceptStart (void)
{
	HLOG (1, "TRFCOMMTool::AcceptStart\n");
	AcceptComplete (noErr);
}

void TRFCOMMTool::TerminateConnection (void)
{
	HLOG (1, "TRFCOMMTool::TerminateConnection\n");
	BluntDisconnectCommand* command = new BluntDisconnectCommand ();
	
	command->fToolPort = * (TObjectId*) ( (Byte*) this + 0x8c);
	memcpy (command->fBdAddr, fPeerBdAddr, 6);
	command->fHCIHandle = fHCIHandle;
	fServerPort.Send (command, sizeof (*command), kNoTimeout, M_COMMAND);
	TCommTool::TerminateConnection ();
}

void TRFCOMMTool::TerminateComplete (void)
{
	HLOG (1, "TRFCOMMTool::TerminateComplete\n");
	TCommTool::TerminateComplete ();
}

#pragma mark -

NewtonErr TRFCOMMTool::PutBytes (CBufferList *list)
{
	HLOG (1, "TRFCOMMTool::PutBytes %d\n", list->GetSize ());
	
	fPutBuffer = list;
	fDataSent = 0;
	SendPendingData ();
	return noErr;
}

NewtonErr TRFCOMMTool::PutFramedBytes (CBufferList *, Boolean)
{
	HLOG (1, "TRFCOMMTool::PutFramedBytes\n");
	return noErr;
}

void TRFCOMMTool::KillPut (void)
{
	HLOG (1, "TRFCOMMTool::KillPut\n");
}

NewtonErr TRFCOMMTool::GetBytes (CBufferList *list)
{
	HLOG (1, "TRFCOMMTool::GetBytes %d\n", list->GetSize ());
	Long n;
	
	fGetBuffer = list;
	if (fSavedDataAmount > 0) {
		HLOG (1, "  Returning %d saved bytes\n", fSavedDataAmount);
		n = fGetBuffer->Putn (fSavedData, fSavedDataAmount);
		if (fSavedDataAmount > n) {
			HLOG (1, "  %d bytes still left\n", fSavedDataAmount - n);
			memcpy (fSavedData, fSavedData + n, fSavedDataAmount - n);
		}
		fSavedDataAmount -= n;
	}
	if (fGetBuffer->AtEOF ()) GetComplete (noErr);
	return noErr;
}

NewtonErr TRFCOMMTool::GetFramedBytes (CBufferList *)
{
	HLOG (1, "TRFCOMMTool::GetFramedBytes\n");
	return noErr;
}

void TRFCOMMTool::KillGet (void)
{
	HLOG (1, "TRFCOMMTool::KillGet\n");
}

void TRFCOMMTool::SendPendingData ()
{
	BluntDataCommand* command = new BluntDataCommand ();

	HLOG (1, "TRFCOMMTool::SendPendingData %d\n", fPutBuffer->GetSize ());
	command->fData = fPutBuffer;
	command->fHCIHandle = fHCIHandle;
	command->fRFCOMMPort = fPeerRFCOMMPort;
	command->fHandler = fDataHandler;
	fServerPort.Send (command, sizeof (*command), kNoTimeout, M_COMMAND);
}
void TRFCOMMTool::Log (int logLevel, char *format, ...)
{
    va_list args;
    char buffer[128];
    
    if (fLogLevel >= logLevel) {
        va_start (args, format);
        vsprintf (buffer, format, args);
        va_end (args);
        _sys_write (1, buffer, strlen (buffer));
    }
}

